@@ -51,7 +51,8 @@ namespace core {
 CurlStack::CurlStack() :
   m_handle((void*)curl_multi_init()),
   m_active(0),
-  m_maxActive(32) {
+  m_maxActive(32),
+  m_ssl_verify_peer(true) {
 
   m_taskTimeout.set_slot(rak::mem_fn(this, &CurlStack::receive_timeout));
 
@@ -164,6 +165,9 @@ CurlStack::add_get(CurlGet* get) {
   if (!m_httpCaCert.empty())
     curl_easy_setopt(get->handle(), CURLOPT_CAINFO, m_httpCaCert.c_str());
 
+  if (!m_ssl_verify_peer)
+    curl_easy_setopt(get->handle(), CURLOPT_SSL_VERIFYPEER, 0); 
+
   base_type::push_back(get);
 
   if (m_active >= m_maxActive)
