@@ -73,8 +73,12 @@ DhtManager::load_dht_cache() {
   if (cache_file.is_open()) {
     cache_file >> cache;
 
-    if (cache_file.fail())
-      throw torrent::input_error("Invalid DHT cache.");
+    // If the cache file is corrupted we will just discard it with an
+    // error message.
+    if (cache_file.fail()) {
+      control->core()->push_log("DHT warning: Cache file corrupted, discarding.");
+      cache = torrent::Object::create_map();
+    }
   }
 
   try {
@@ -99,7 +103,7 @@ DhtManager::start_dht() {
   torrent::dht_manager()->set_upload_throttle(throttles.first);
   torrent::dht_manager()->set_download_throttle(throttles.second);
 
-  int port = rpc::call_command_value("get_dht_port");
+  int port = rpc::call_command_value("dht.port");
   if (port <= 0)
     return;
 
@@ -163,7 +167,7 @@ DhtManager::save_dht_cache() {
 }
 
 void
-DhtManager::set_start(const std::string& arg) {
+DhtManager::set_mode(const std::string& arg) {
   int i;
   for (i = 0; i < dht_settings_num; i++) {
     if (arg == dht_settings[i]) {
@@ -190,7 +194,7 @@ DhtManager::update() {
     DownloadList::const_iterator itr, end;
 
     for (itr = control->core()->download_list()->begin(), end = control->core()->download_list()->end(); itr != end; ++itr)
-      if ((*itr)->is_active() && !(*itr)->download()->is_private())
+      if ((*itr)->download()->info()->is_active() && !(*itr)->download()->info()->is_private())
         break;
       
     if (itr == end) {
@@ -296,6 +300,8 @@ DhtManager::dht_statistics() {
     dhtStats.insert_key("queries_received", stats.queries_received);
     dhtStats.insert_key("queries_sent",     stats.queries_sent);
     dhtStats.insert_key("replies_received", stats.replies_received);
+    dhtStats.insert_key("errors_received",  stats.errors_received);
+    dhtStats.insert_key("errors_caught",    stats.errors_caught);
     dhtStats.insert_key("bytes_read",       stats.down_rate.total());
     dhtStats.insert_key("bytes_written",    stats.up_rate.total());
     dhtStats.insert_key("nodes",            stats.num_nodes);
