@@ -63,18 +63,18 @@ Download::Download(download_type d) :
   m_chunksFailed(0),
   m_resumeFlags(~uint32_t()) {
 
-  m_connTrackerSucceded = m_download.signal_tracker_succeded(sigc::bind(sigc::mem_fun(*this, &Download::receive_tracker_msg), ""));
-  m_connTrackerFailed   = m_download.signal_tracker_failed(sigc::mem_fun(*this, &Download::receive_tracker_msg));
-  m_connStorageError    = m_download.signal_storage_error(sigc::mem_fun(*this, &Download::receive_storage_error));
+  m_connTrackerSucceeded = m_download.info()->signal_tracker_success().connect(sigc::bind(sigc::mem_fun(*this, &Download::receive_tracker_msg), ""));
+  m_connTrackerFailed   = m_download.info()->signal_tracker_failed().connect(sigc::mem_fun(*this, &Download::receive_tracker_msg));
+  m_connStorageError    = m_download.info()->signal_storage_error().connect(sigc::mem_fun(*this, &Download::receive_storage_error));
 
-  m_download.signal_chunk_failed(sigc::mem_fun(*this, &Download::receive_chunk_failed));
+  m_download.info()->signal_chunk_failed().connect(sigc::mem_fun(*this, &Download::receive_chunk_failed));
 }
 
 Download::~Download() {
   if (!m_download.is_valid())
     return;
 
-  m_connTrackerSucceded.disconnect();
+  m_connTrackerSucceeded.disconnect();
   m_connTrackerFailed.disconnect();
   m_connStorageError.disconnect();
 
@@ -159,7 +159,7 @@ Download::receive_chunk_failed(__UNUSED uint32_t idx) {
 
 void
 Download::set_throttle_name(const std::string& throttleName) {
-  if (m_download.is_active())
+  if (m_download.info()->is_active())
     throw torrent::input_error("Cannot set throttle on active download.");
 
   torrent::ThrottlePair throttles = control->core()->get_throttle(throttleName);
