@@ -87,6 +87,7 @@ public:
 
   // Might move this to DownloadFactory.
   Download*           create(std::istream* str, bool printLog);
+  Download*           create(torrent::Object* obj, bool printLog);
 
   iterator            insert(Download* d);
 
@@ -161,6 +162,8 @@ private:
 
   void                received_finished(Download* d);
   void                confirm_finished(Download* d);
+
+  void                process_meta_download(Download* d);
 };
 
 }
