@@ -102,7 +102,7 @@ ViewManager::sort(const std::string& name, uint32_t timeout) {
 }
 
 void
-ViewManager::set_filter(const std::string& name, const std::string& cmd) {
+ViewManager::set_filter(const std::string& name, const torrent::Object& cmd) {
   iterator viewItr = find_throw(name);
 
   (*viewItr)->set_filter(cmd);
@@ -115,6 +115,8 @@ ViewManager::set_filter_on(const std::string& name, const filter_args& args) {
 
   (*viewItr)->clear_filter_on();
 
+  // TODO: Ensure the filter keys are rlookup.
+
   for (filter_args::const_iterator itr = args.begin(); itr != args.end(); ++itr)
     (*viewItr)->set_filter_on_event(*itr);
 }
