@@ -65,7 +65,7 @@ public:
   int                 attributes() const            { return m_attributes; }
   void                set_attributes(int a)         { m_attributes = a; }
 
-  virtual char*       print(char* first, char* last, Canvas::attributes_list* attributes, void* object);
+  virtual char*       print(char* first, char* last, Canvas::attributes_list* attributes, rpc::target_type target);
 
 protected:
   virtual int64_t     value(void* object) = 0;
