@@ -43,6 +43,7 @@
 #include "thread_base.h"
 #include "thread_main.h"
 #include "thread_worker.h"
+#include "rpc/ip_table_list.h"
 
 class Control;
 
@@ -53,6 +54,7 @@ class Control;
 
 extern rak::priority_queue_default taskScheduler;
 extern rak::timer                  cachedTime;
+extern rpc::ip_table_list          ip_tables;
 
 extern Control*      control;
 // extern __thread ThreadBase* main_thread; // Only use for worker threads for now.
