@@ -97,7 +97,24 @@ CommandScheduler::call_item(value_type item) {
   // removed.
 
   try {
-    rpc::parse_command_multiple_std(item->command());
+    if (item->command().is_string()) {
+      rpc::parse_command_multiple_std(item->command().as_string());
+
+    } else if (item->command().is_dict_key()) {
+      // This can/should be optimized...
+      torrent::Object tmp_command = item->command();
+
+      // Unquote the root function object so 'parse_command_execute'
+      // doesn't end up calling it.
+      //
+      // TODO: Only call this if mask_function is set?
+      uint32_t flags = tmp_command.flags() & torrent::Object::mask_function;
+      tmp_command.unset_flags(torrent::Object::mask_function);
+      tmp_command.set_flags((flags >> 1) & torrent::Object::mask_function);
+
+      rpc::parse_command_execute(rpc::make_target(), &tmp_command);
+      rpc::commands.call_command(tmp_command.as_dict_key().c_str(), tmp_command.as_dict_obj());
+    }
 
   } catch (torrent::input_error& e) {
     if (m_slotErrorMessage.is_valid())
@@ -119,13 +136,19 @@ CommandScheduler::call_item(value_type item) {
 }
 
 void
-CommandScheduler::parse(const std::string& key, const std::string& bufAbsolute, const std::string& bufInterval, const std::string& command) {
+CommandScheduler::parse(const std::string& key,
+                        const std::string& bufAbsolute,
+                        const std::string& bufInterval,
+                        const torrent::Object& command) {
+  if (!command.is_string() && !command.is_dict_key())
+    throw torrent::bencode_error("Invalid type passed to command scheduler.");
+
   uint32_t absolute = parse_absolute(bufAbsolute.c_str());
   uint32_t interval = parse_interval(bufInterval.c_str());
 
   CommandSchedulerItem* item = *insert(key);
 
-  item->set_command(command);
+  item->command() = command;
   item->set_interval(interval);
 
   item->enable((cachedTime + rak::timer::from_seconds(absolute)).round_seconds());
