@@ -38,6 +38,7 @@
 
 #include "object_storage.h"
 
+#include "rak/functional.h"
 #include "parse.h"
 #include "parse_commands.h"
 
@@ -62,26 +63,31 @@ object_storage::insert(const char* key_data, uint32_t key_size, const torrent::O
   // Check for size > key_size.
   // Check for empty string.
 
+  bool use_raw = false;
   torrent::Object object;
 
   switch (flags & mask_type) {
   case flag_bool_type:     object = !!convert_to_value(rawObject); break;
   case flag_value_type:    object = convert_to_value(rawObject); break;
   case flag_string_type:   object = convert_to_string(rawObject); break;
-  case flag_function_type: object = convert_to_string(rawObject); break;
+  case flag_list_type:     use_raw = true; break;
+  case flag_function_type: use_raw = true; break;
   case flag_multi_type:    object = torrent::Object::create_map(); break;
   }
 
   if (!(flags & mask_type))
     throw torrent::input_error("No type flags set when calling object_storage::insert.");
 
+  if ((flags & flag_rlookup) && (!(flags & flag_static) || !(flags & flag_multi_type)))
+    throw torrent::input_error("Cannot insert non-static or non-multi-type object with rlookup enabled.");
+
   std::pair<iterator, bool> result = base_type::insert(std::make_pair(key_type(key_data, key_size), object_storage_node()));
 
   if (!result.second)
     throw torrent::input_error("Key already exists in object_storage.");
 
   result.first->second.flags = flags;
-  result.first->second.object = object;
+  result.first->second.object = use_raw ? rawObject : object;
 
   return result.first;
 }
@@ -128,6 +134,16 @@ object_storage::set_string(const torrent::raw_string& key, const std::string& ob
   return itr->second.object = object;
 }
 
+const torrent::Object&
+object_storage::set_list(const torrent::raw_string& key, const torrent::Object::list_type& object) {
+  local_iterator itr = find_local(key);
+
+  if (itr == end(bucket_count()) || (itr->second.flags & mask_type) != flag_list_type)
+    throw torrent::input_error("Key not found or wrong type.");
+
+  return itr->second.object = torrent::Object::create_list_range(object.begin(), object.end());
+}
+
 const torrent::Object&
 object_storage::set_function(const torrent::raw_string& key, const std::string& object) {
   local_iterator itr = find_local(key);
@@ -147,7 +163,7 @@ object_storage::call_function(const torrent::raw_string& key, target_type target
 
   switch (itr->second.flags & mask_type) {
   case flag_function_type:
-    return command_function_call_str(itr->second.object.as_string(), target, object);
+    return command_function_call_object(itr->second.object, target, object);
   case flag_multi_type:
     return command_function_multi_call(itr->second.object.as_map(), target, object);
   default:
@@ -171,16 +187,71 @@ object_storage::erase_multi_key(const torrent::raw_string& key, const std::strin
     return;
 
   itr->second.object.erase_key(cmd_key);
+
+  if (!(itr->second.flags & flag_rlookup))
+    return;
+
+  // Remove the rlookup entry.
+  rlookup_iterator r_itr = m_rlookup.find(cmd_key);
+
+  if (r_itr == m_rlookup.end())
+    return;
+
+  rlookup_mapped_iterator rm_itr = std::find_if(r_itr->second.begin(), r_itr->second.end(),
+                                                rak::equal(key, rak::mem_ptr(&value_type::first)));
+
+  if (rm_itr != r_itr->second.end())
+    r_itr->second.erase(rm_itr);
 }
 
 void
-object_storage::set_multi_key(const torrent::raw_string& key, const std::string& cmd_key, const std::string& object) {
+object_storage::set_multi_key_obj(const torrent::raw_string& key, const std::string& cmd_key, const torrent::Object& object) {
+  if (!object.is_string() && !object.is_dict_key())
+    throw torrent::input_error("Object is wrong type.");
+
   local_iterator itr = find_local(key);
 
   if (itr == end(0) || (itr->second.flags & mask_type) != flag_multi_type)
     throw torrent::input_error("Key not found or wrong type.");
 
+  if (itr->second.flags & flag_rlookup) {
+    rlookup_iterator r_itr = m_rlookup.find(cmd_key);
+
+    if (r_itr == m_rlookup.end())
+      r_itr = m_rlookup.insert(std::make_pair(cmd_key, rlookup_type::mapped_type())).first;
+
+    if (std::find_if(r_itr->second.begin(), r_itr->second.end(),
+                     rak::equal(key, rak::mem_ptr(&value_type::first))) == r_itr->second.end())
+      r_itr->second.push_back(&*itr);
+  }
+
   itr->second.object.insert_key(cmd_key, object);
 }
 
+torrent::Object::list_type
+object_storage::rlookup_list(const std::string& cmd_key) {
+  torrent::Object::list_type result;
+
+  rlookup_iterator r_itr = m_rlookup.find(cmd_key);
+  
+  if (r_itr != m_rlookup.end())
+    std::transform(r_itr->second.begin(), r_itr->second.end(), std::back_inserter(result),
+                   std::bind(&key_type::c_str, std::bind(rak::mem_ptr(&value_type::first), std::placeholders::_1)));
+
+  return result;
+}
+
+void
+object_storage::rlookup_clear(const std::string& cmd_key) {
+  rlookup_iterator r_itr = m_rlookup.find(cmd_key);
+  
+  if (r_itr == m_rlookup.end())
+    return;
+
+  for (rlookup_mapped_iterator first = r_itr->second.begin(), last = r_itr->second.end(); first != last; first++)
+    (*first)->second.object.erase_key(cmd_key);
+
+  r_itr->second.clear();
+}
+
 }
