@@ -62,6 +62,8 @@ typedef std::pair<torrent::Object, const char*> parse_command_type;
 parse_command_type     parse_command(target_type target, const char* first, const char* last);
 torrent::Object        parse_command_multiple(target_type target, const char* first, const char* last);
 
+void                   parse_command_execute(target_type target, torrent::Object* object);
+
 // Make this take care of lists too.
 parse_command_type     parse_command_object(target_type target, const torrent::Object& object);
 
@@ -76,6 +78,11 @@ parse_command_single(target_type target, const std::string& cmd) {
   return parse_command(target, cmd.c_str(), cmd.c_str() + cmd.size()).first;
 }
 
+inline torrent::Object
+parse_command_multiple_std(const std::string& cmd, target_type target = rpc::make_target()) {
+  return parse_command_multiple(target, cmd.c_str(), cmd.c_str() + cmd.size());
+}
+
 inline void
 parse_command_single_std(const std::string& cmd) {
   parse_command(make_target(), cmd.c_str(), cmd.c_str() + cmd.size());
@@ -91,22 +98,6 @@ parse_command_multiple_d_nothrow(core::Download* download, const std::string& cm
   }
 }
 
-inline void
-parse_command_d_single_std(core::Download* download, const std::string& cmd) {
-  parse_command(make_target(download), cmd.c_str(), cmd.c_str() + cmd.size());
-}
-
-inline torrent::Object
-parse_command_multiple_std(const std::string& cmd) {
-  return parse_command_multiple(make_target(), cmd.c_str(), cmd.c_str() + cmd.size());
-}
-
-inline torrent::Object
-parse_command_d_multiple_std(core::Download* download, const std::string& cmd) {
-  return parse_command_multiple(make_target(download), cmd.c_str(), cmd.c_str() + cmd.size());
-}
-
-// inline torrent::Object call_command(const char* key, const torrent::Object& obj, target_type target = target_type((int)CommandMap::target_generic, NULL)) { return commands.call_command(key, obj); }
 inline torrent::Object call_command       (const char* key, const torrent::Object& obj, target_type target = make_target()) { return commands.call_command(key, obj, target); }
 inline torrent::Object call_command_void  (const char* key, target_type target = make_target()) { return commands.call_command(key, torrent::Object(), target); }
 inline std::string     call_command_string(const char* key, target_type target = make_target()) { return commands.call_command(key, torrent::Object(), target).as_string(); }
@@ -141,6 +132,8 @@ call_command_d_range(const char* key, core::Download* download, torrent::Object:
 const torrent::Object
 command_function_call(const torrent::raw_string& cmd, target_type target, const torrent::Object& args);
 const torrent::Object
+command_function_call_object(const torrent::Object& cmd, target_type target, const torrent::Object& args);
+const torrent::Object
 command_function_multi_call(const torrent::Object::map_type& cmd, target_type target, const torrent::Object& args);
 
 inline const torrent::Object
