@@ -37,7 +37,7 @@
 #ifndef RTORRENT_UI_ELEMENT_TEXT_H
 #define RTORRENT_UI_ELEMENT_TEXT_H
 
-#include <sigc++/slot.h>
+#include <sigc++/functors/slot.h>
 
 #include "core/download.h"
 #include "display/text_element_string.h"
@@ -60,11 +60,11 @@ public:
   typedef uint32_t                    size_type;
   typedef uint32_t                    extent_type;
 
-  ElementText(void *object);
+  ElementText(rpc::target_type target);
   ~ElementText();
 
-  void*               object() const           { return m_window->object(); }
-  void                set_object(void* object) { m_window->set_object(object); m_window->mark_dirty(); }
+  rpc::target_type    target() const                      { return m_window->target(); }
+  void                set_target(rpc::target_type target) { m_window->set_target(target); m_window->mark_dirty(); }
 
   uint32_t            interval() const         { return m_window->interval(); }
   void                set_interval(uint32_t i) { m_window->set_interval(i); m_window->mark_dirty(); }
@@ -95,10 +95,11 @@ public:
                                   text_element_wrapper entry3, text_element_wrapper entry4,
                                   text_element_wrapper entry5, text_element_wrapper entry6);
 
-  void                set_column(unsigned int column)     { m_column = column; }
+  void                set_column(unsigned int column)            { m_column = column; }
+  void                set_error_handler(display::TextElement* t) { m_window->set_error_handler(t); }
 
-  extent_type         column_width() const                { return m_columnWidth; }
-  void                set_column_width(extent_type width) { m_columnWidth = width; }
+  extent_type         column_width() const                       { return m_columnWidth; }
+  void                set_column_width(extent_type width)        { m_columnWidth = width; }
 
 private:
   WindowText*         m_window;
